/*
 * Created on Jun 7, 2004 
 * 
 *  Copyright 2004, by the California Institute of Technology. ALL RIGHTS RESERVED. 
 *  United States Government Sponsorship acknowledged. Any commercial use must be 
 *  negotiated with the Office of Technology Transfer at the California Institute 
 *  of Technology. 
 *
 *  This software may be subject to U.S. export control laws and regulations.  
 *  By accepting this document, the user agrees to comply with all applicable U.S. 
 *  export laws and regulations.  User has the responsibility to obtain export 
 *  licenses, or other export authority as may be required before exporting such 
 *  information to foreign countries or providing access to foreign persons. 
 */
package suramadu.util.exec;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * @author dan To change the template for this generated type comment go to Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code
 *         and Comments
 */
public class MemoryType {
    static class MemoryTypeException extends RuntimeException {
        MemoryTypeException() {
            super();
        }

        MemoryTypeException( String msg ) {
            super( msg );
        }
    }

    /** Memory type enumerations */
    public static final MemoryType HEAPMEMORY     = new MemoryType();
    public static final MemoryType IMMORTALMEMORY = new MemoryType();
    public static final MemoryType SCOPEMEMORY    = new MemoryType();

    /** property value map to memory types */
    private static Map             MEMTYPEMAP     = new HashMap();
    static {
        MEMTYPEMAP.put( "HEAP", MemoryType.HEAPMEMORY );
        MEMTYPEMAP.put( "IMMORTAL", MemoryType.IMMORTALMEMORY );
        MEMTYPEMAP.put( "SCOPE", MemoryType.SCOPEMEMORY );
    }
    
    public static MemoryType getMemoryType( String memoryType ) {
        MemoryType mt = (MemoryType)MEMTYPEMAP.get( memoryType.trim().toUpperCase() );

        if ( null == mt ) {
            StringBuffer mb = new StringBuffer();
            
            mb.append( "memory type must be: " );
            int count = 0;
            for ( Iterator i = MemoryType.getTypeNames(); i.hasNext(); count++ ) {
                if ( 0 != count ) mb.append( ", " );
                mb.append( "\"" + i.next() + "\"" );
            }
            mb.append( ", and not \"" + memoryType + "\"" );

            throw new MemoryType.MemoryTypeException( mb.toString() );
        }
        
        return mt;
    }
    
    public static Iterator getTypeNames() {
        return MEMTYPEMAP.keySet().iterator();
    }

    private static int             nextID         = 0;
    
    private static String[] TYPEINDEX = new String[ 3 ];
    static {
        for ( Iterator i = getTypeNames(); i.hasNext(); ) {
            String     mname = (String)i.next();
            MemoryType mt    = getMemoryType( mname );
            TYPEINDEX[ mt.id ] = mname;
        }
    }

    /**
     * 
     * @uml.property name="id"
     */
    private int id;


    protected MemoryType() {
        assignID();
    }

    private synchronized void assignID() {
        id = nextID++;
    }

    /**
     * 
     * @uml.property name="id"
     */
    public int getId() {
        return id;
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     */
    public boolean equals( Object obj ) {
        if ( obj instanceof MemoryType ) {
            return (id == ((MemoryType)obj).getId());
        }
        else
            return false;
    }

    public String toString() {
        return super.toString() + ": " + TYPEINDEX[ id ] + " (id=" + id + ")";
    }
}