/*
 * Launcher.java Created on August 6, 2003, 2:03 PM
 * 
 *  Copyright 2004, by the California Institute of Technology. ALL RIGHTS RESERVED. 
 *  United States Government Sponsorship acknowledged. Any commercial use must be 
 *  negotiated with the Office of Technology Transfer at the California Institute 
 *  of Technology. 
 *
 *  This software may be subject to U.S. export control laws and regulations.  
 *  By accepting this document, the user agrees to comply with all applicable U.S. 
 *  export laws and regulations.  User has the responsibility to obtain export 
 *  licenses, or other export authority as may be required before exporting such 
 *  information to foreign countries or providing access to foreign persons. 
 */

package suramadu.util.exec;

import java.io.PrintWriter;
import java.io.StringWriter;

import javax.realtime.AperiodicParameters;
import javax.realtime.AsyncEventHandler;
import javax.realtime.HeapMemory;
import javax.realtime.ImmortalMemory;
import javax.realtime.LTMemory;
import javax.realtime.MemoryArea;
import javax.realtime.MemoryParameters;
import javax.realtime.PeriodicParameters;
import javax.realtime.PriorityParameters;
import javax.realtime.PriorityScheduler;
import javax.realtime.ProcessingGroupParameters;
import javax.realtime.RealtimeThread;
import javax.realtime.RelativeTime;
import javax.realtime.ReleaseParameters;
import javax.realtime.SchedulingParameters;

/**
 * Utility used to launch realtime threads. It requires a Runnable object the type of thread you want to run in and the priority to
 * run at. It will create the Thread, SchedulingParameters, ReleaseParameters MemoryParameters MemoryArea and
 * ProcessingGroupParameters. It will the start the thread.
 * 
 * @author mbi,briang
 * @version $Id: RTThreadLauncher.java,v 1.1 2004/09/23 15:34:23 mbi Exp $
 */
public class RTThreadLauncher extends JavaThreadLauncher {
    public static final int   DEFAULT_WRAPPER_SCRATCH_SIZE = 16 * 1024 * 1024;	// 16Mb Default Wrapper Scratch Size.
    public static final int   DEFAULT_SCOPE_SIZE           = 60 * 1024 * 1024;  // 60Mb Default Scope Size.

    /**
     * 
     * @uml.property name="rtOverrunHandler"
     * @uml.associationEnd 
     * @uml.property name="rtOverrunHandler" multiplicity="(0 1)"
     */
    private AsyncEventHandler rtOverrunHandler = null;

    /**
     * 
     * @uml.property name="rtMissHandler"
     * @uml.associationEnd 
     * @uml.property name="rtMissHandler" multiplicity="(0 1)"
     */
    private AsyncEventHandler rtMissHandler = null;

    /**
     * 
     * @uml.property name="rtCost"
     * @uml.associationEnd 
     * @uml.property name="rtCost" multiplicity="(0 1)"
     */
    private RelativeTime rtCost = null;

    /**
     * 
     * @uml.property name="rtDeadline"
     * @uml.associationEnd 
     * @uml.property name="rtDeadline" multiplicity="(0 1)"
     */
    private RelativeTime rtDeadline = null;

    /**
     * 
     * @uml.property name="wrapperScratchSize"
     */
    private int wrapperScratchSize = DEFAULT_WRAPPER_SCRATCH_SIZE;

    /**
     * 
     * @uml.property name="scopeSize"
     */
    private int scopeSize = DEFAULT_SCOPE_SIZE;


    /**
     * Creates a new instance of Launcher
     */
    public RTThreadLauncher() {
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param data
     *            initialize from data structure
     */
    public RTThreadLauncher( ThreadLauncherData data ) {
        super( data );
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param runnable
     *            the object to run in the created thread. The deafult Launcher.ThreadType is REALTIMETHREAD the deafault MemoryArea
     *            is Heap and the deafult priority is PriorityScheduler.MAX_PRIORITY.
     */
    public RTThreadLauncher( Runnable runnable ) {
        super( runnable );
        this.runnable = runnable;
        priority = PriorityScheduler.MAX_PRIORITY;
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param runnable
     *            the object to run in the created thread. The deafult Launcher.ThreadType is REALTIMETHREAD the deafault MemoryArea
     *            is Heap and the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     */
    public RTThreadLauncher( Runnable runnable, int priority ) {
        super( runnable, priority );
        this.runnable = runnable;
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param runnable
     *            the object to run in the created thread. The deafult Launcher.ThreadType is REALTIMETHREAD the deafault MemoryArea
     *            is Heap and the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     */
    public RTThreadLauncher( Runnable runnable, int priority, long period ) {
        super( runnable, priority, period );
        this.runnable = runnable;
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param runnable
     *            the object to run in the created thread. The deafult Launcher.ThreadType is REALTIMETHREAD the deafault MemoryArea
     *            is Heap and the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     * @param initialDelay
     *            the initial delay before beginning periodic execution in milliseconds.
     */
    public RTThreadLauncher( Runnable runnable, int priority, long period, long initialDelay ) {
        super( runnable, priority, period, initialDelay );
        this.runnable = runnable;
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param tt
     *            the ThreadType to use for this launcher
     * @param runnable
     *            the object to run in the created thread. The deafult Launcher.ThreadType is REALTIMETHREAD the deafault MemoryArea
     *            is Heap and the deafult priority is PriorityScheduler.MAX_PRIORITY.
     */
    public RTThreadLauncher( ThreadType tt, Runnable runnable ) {
        super( tt, runnable );
        this.runnable = runnable;
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param tt
     *            the ThreadType to use for this launcher
     * @param runnable
     *            the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     */
    public RTThreadLauncher( ThreadType tt, Runnable runnable, int priority ) {
        super( tt, runnable, priority );
        this.runnable = runnable;
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param tt
     *            the ThreadType to use for this launcher
     * @param runnable
     *            the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     */
    public RTThreadLauncher( ThreadType tt, Runnable runnable, int priority, long period ) {
        super( tt, runnable, priority, period );
        this.runnable = runnable;
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param tt
     *            the ThreadType to use for this launcher
     * @param runnable
     *            the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     * @param initialDelay
     *            the initial delay before beginning periodic execution in milliseconds.
     */
    public RTThreadLauncher( ThreadType tt, Runnable runnable, int priority, long period, long initialDelay ) {
        super( tt, runnable, priority, period, initialDelay );
        this.runnable = runnable;
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param tt
     *            the ThreadType to use for this launcher
     * @param mt
     *            the MemoryType to use for this launcher
     * @param runnable
     *            the object to run in the created thread. The deafult Launcher.ThreadType is REALTIMETHREAD the deafault MemoryArea
     *            is Heap and the deafult priority is PriorityScheduler.MAX_PRIORITY.
     */
    public RTThreadLauncher( ThreadType tt, MemoryType mt, Runnable runnable ) {
        super( tt, mt, runnable );
        this.runnable = runnable;
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param tt
     *            the ThreadType to use for this launcher
     * @param mt
     *            the MemoryType to use for this launcher
     * @param runnable
     *            the object to run in the created thread. The deafult Launcher.ThreadType is REALTIMETHREAD the deafault MemoryArea
     *            is Heap and the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     */
    public RTThreadLauncher( ThreadType tt, MemoryType mt, Runnable runnable, int priority ) {
        super( tt, mt, runnable, priority );
        this.runnable = runnable;
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param tt
     *            the ThreadType to use for this launcher
     * @param mt
     *            the MemoryType to use for this launcher
     * @param runnable
     *            the object to run in the created thread. The deafult Launcher.ThreadType is REALTIMETHREAD the deafault MemoryArea
     *            is Heap and the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     */
    public RTThreadLauncher( ThreadType tt, MemoryType mt, Runnable runnable, int priority, long period ) {
        super( tt, mt, runnable, priority, period );
        this.runnable = runnable;
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param tt
     *            the ThreadType to use for this launcher
     * @param mt
     *            the MemoryType to use for this launcher
     * @param runnable
     *            the object to run in the created thread. The deafult Launcher.ThreadType is REALTIMETHREAD the deafault MemoryArea
     *            is Heap and the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     * @param initialDelay
     *            the initial delay before beginning periodic execution in milliseconds.
     */
    public RTThreadLauncher( ThreadType tt, MemoryType mt, Runnable runnable, int priority, long period, long initialDelay ) {
        super( tt, mt, runnable, priority, period, initialDelay );
        this.runnable = runnable;
    }

    /**
     * Kill the launched thread appropriately for its type.
     */
    public void kill() {
        try {
            ((RTJavaThread)thread).kill();
            return;
        }
        catch ( ClassCastException e ) {
        }

        try {
            ((NHRTJavaThread)thread).kill();
            return;
        }
        catch ( ClassCastException e ) {
        }

        try {
            super.kill();
            return;
        }
        catch ( ClassCastException e ) {
        }
    }

    public void setThreadType( ThreadType tt ) {
        if ( ThreadType.JAVATHREAD.equals( tt ) || ThreadType.PERIODICJAVATHREAD.equals( tt ) ) {
            throw new RuntimeException( "Cannot set thread type to: " + tt + " for RTThreadLauncher." );
        }
        super.setThreadType( tt );
    }

    /**
     * 
     * @uml.property name="wrapperScratchSize"
     */
    public void setWrapperScratchSize(int size) {
        wrapperScratchSize = size;
    }

    /**
     * 
     * @uml.property name="scopeSize"
     */
    public void setScopeSize(int size) {
        scopeSize = size;
    }



    public void setPeriod( long p ) {
        period = p;
    }

    public void setInitialDelay( long d ) {
        iDelay = d;
    }

    public void setOverrunHandler( AsyncEventHandler rtOverrunHandler ) {
        this.rtOverrunHandler = rtOverrunHandler;
    }

    public AsyncEventHandler getOverrunHandler() {
        return rtOverrunHandler;
    }

    public void setMissHandler( AsyncEventHandler rtMissHandler ) {
        this.rtMissHandler = rtMissHandler;
    }

    /**
     * 
     * @uml.property name="wrapperScratchSize"
     */
    public int getWrapperScratchSize() {
        return wrapperScratchSize;
    }

    /**
     * 
     * @uml.property name="scopeSize"
     */
    public int getScopeSize() {
        return scopeSize;
    }

    public AsyncEventHandler getMissHandler() {
        return rtMissHandler;
    }

    public void setRTCost( RelativeTime rtCost ) {
        this.rtCost = rtCost;
    }

    public RelativeTime getRTCost() {
        return rtCost;
    }

    public void setRTDeadline( RelativeTime rtDeadline ) {
        this.rtDeadline = rtDeadline;
    }

    public RelativeTime getRTDeadline() {
        return rtDeadline;
    }

    /**
     * Creates the Thread, SchedulingParameters, ReleaseParameters MemoryParameters MemoryArea and ProcessingGroupParameters. This
     * method may be called by any thread type in any memory area. If we are creating a regular Java thread, the memory type request
     * is checked for validity for that thread type, and if it is OK, then the superclass' create() method is called. If we are
     * creating a RealtimeThread, then we create a wrapper thread
     */
    protected Thread create() {
        if ( threadType.equals( ThreadType.JAVATHREAD ) || threadType.equals( ThreadType.PERIODICJAVATHREAD ) ) {
            if ( memoryType.equals( MemoryType.IMMORTALMEMORY ) || memoryType.equals( MemoryType.SCOPEMEMORY ) ) {
                throw new RuntimeException( "Cannot use IMMORTAL or SCOPE memory with JAVA or PERIODICJAVA threads." );
            }
            thread = super.create();
        }

        /*
         * Run creation thread in immortal memory
         */
        else {
            RealtimeThread initializer = new RealtimeThread( null, null, null, ImmortalMemory.instance(), null, new Runnable() {
                public void run() {
                    MemoryArea curMemArea = RealtimeThread.getCurrentMemoryArea();
                    MemoryArea scratchpad = null;
                    
                    if ( !(curMemArea instanceof ImmortalMemory ) ) {
                        throw new RuntimeException( "Internal Error: RT Initialization running in: " + curMemArea );
                    }

                    /*
                     * RUNNING IN IMMORTAL NOW!
                     */

                    /*
                     * Allocate scratchpad memory for initialization
                     */
                    if ( memoryType.equals( MemoryType.IMMORTALMEMORY ) ) {
                        scratchpad = ImmortalMemory.instance(); // Immortal scratchpad
                    }
                    else if ( memoryType.equals( MemoryType.SCOPEMEMORY ) ) {
                        scratchpad = ImmortalMemory.instance();
//                      scratchpad = new LTMemory( wrapperScratchSize, wrapperScratchSize );
                    }
                    else {
                        scratchpad = HeapMemory.instance(); // Heap scratchpad
                    }

                    /*
                     * Create a NoHeapRealtimeThread
                     */
                    if ( threadType.equals( ThreadType.NOHEAPREALTIMETHREAD ) ) {
                        Thread initThread = new RealtimeThread( null, null, null, scratchpad, null, new Runnable() {
                            /**
                             * Allocated in IMMORTAL RTSJ specific data for defining periodic and aperiodic thread behavior
                             */
                            public void run() {
                                SchedulingParameters schedulingParameters = null;
                                ReleaseParameters releaseParameters = null;
                                MemoryArea memoryArea = null;
                                MemoryParameters memoryParameters = null;
                                ProcessingGroupParameters processingGroupParameters = null;

                                /*
                                 * RUNNING IN SCOPE SCRATCHPAD NOW!
                                 */

                                // set SchedulingParameters
                                schedulingParameters = new PriorityParameters( priority );

                                // set ReleaseParameters
                                if ( 0 < period ) {
                                    RelativeTime rtStart = new RelativeTime( getInitialDelay(), 0 );
                                    RelativeTime rtPeriod = new RelativeTime( getPeriod(), 0 );
                                    releaseParameters = new PeriodicParameters( rtStart, rtPeriod, rtCost, rtDeadline, rtOverrunHandler, rtMissHandler );
                                }
                                else {
                                    releaseParameters = new AperiodicParameters( rtCost, rtDeadline, rtOverrunHandler, rtMissHandler );
                                }

                                // set MemoryArea
                                if ( memoryType.equals( MemoryType.HEAPMEMORY ) ) {
                                    throw new RuntimeException( "Cannot specify NHRT with HEAP" );
                                }
                                else if ( memoryType.equals( MemoryType.IMMORTALMEMORY ) ) {
                                    memoryArea = ImmortalMemory.instance();
                                }
                                else if ( memoryType.equals( MemoryType.SCOPEMEMORY ) ) {
                                    memoryArea = new LTMemory( scopeSize, scopeSize );
                                }

                                // run initialization specific to test.
                                try {
                                    if ( runnable instanceof TestInitializer ) {
                                        runnable.run();
                                        runnable = ((TestInitializer)runnable).getTestRunner();
                                    }

                                    thread = new NHRTJavaThread( schedulingParameters, releaseParameters, memoryParameters, memoryArea, processingGroupParameters, runnable, iterations );
                                }
                                catch ( Exception e ) {
                                    throw new RuntimeException( "Error starting NoHeapRealtimeThread: " + e );
                                }
                            }
                        } );
                        try {
                            initThread.start();
                            initThread.join();

                        }
                        catch ( InterruptedException e ) {
                            e.printStackTrace();
                        }
                    }

                    /*
                     * Create a RealtimeThread
                     */
                    else if ( threadType.equals( ThreadType.REALTIMETHREAD ) ) {
                        Thread initThread = new RealtimeThread( null, null, null, scratchpad, null, new Runnable() {
                            /**
                             * Allocated in IMMORTAL RTSJ specific data for defining periodic and aperiodic thread behavior
                             */
                            public void run() {
                                SchedulingParameters schedulingParameters = null;
                                ReleaseParameters releaseParameters = null;
                                MemoryArea memoryArea = null;
                                MemoryParameters memoryParameters = null;
                                ProcessingGroupParameters processingGroupParameters = null;

                                /*
                                 * RUNNING IN SCOPE SCRATCHPAD NOW!
                                 */

                                // set SchedulingParameters
                                schedulingParameters = new PriorityParameters( priority );

                                // set ReleaseParameters
                                if ( 0 < period ) {
                                    RelativeTime rtStart = new RelativeTime( getInitialDelay(), 0 );
                                    RelativeTime rtPeriod = new RelativeTime( getPeriod(), 0 );
                                    releaseParameters = new PeriodicParameters( rtStart, rtPeriod, rtCost, rtDeadline, rtOverrunHandler, rtMissHandler );
                                }
                                else {
                                    releaseParameters = new AperiodicParameters( rtCost, rtDeadline, rtOverrunHandler, rtMissHandler );
                                }

                                // set MemoryArea
                                if ( memoryType.equals( MemoryType.HEAPMEMORY ) ) {
                                    memoryArea = HeapMemory.instance();
                                }
                                else if ( memoryType.equals( MemoryType.IMMORTALMEMORY ) ) {
                                    memoryArea = ImmortalMemory.instance();
                                }
                                else if ( memoryType.equals( MemoryType.SCOPEMEMORY ) ) {
                                    memoryArea = new LTMemory( scopeSize, scopeSize );
                                }

                                // run initialization specific to test.
                                try {
                                    if ( runnable instanceof TestInitializer ) {
                                        runnable.run();
                                        runnable = ((TestInitializer)runnable).getTestRunner();
                                    }

                                    thread = new RTJavaThread( schedulingParameters, releaseParameters, memoryParameters, memoryArea, processingGroupParameters, runnable, iterations );
                                }
                                catch ( Exception e ) {
                                    throw new RuntimeException( "Error starting RealtimeThread: " + e );
                                }
                            }
                        } );
                        try {
                            initThread.start();
                            initThread.join();
                        }
                        catch ( InterruptedException e ) {
                            e.printStackTrace();
                        }
                    }
                }
            } );
            try {
                initializer.start();
                initializer.join();
            }
            catch ( InterruptedException e ) {
                e.printStackTrace();
            }
        }

        /*
         * Create regular Java thread.
         */
        return thread;
    }
    
    /**
     * @return a string representing the contents of this class.
     */
    public String toString() {
        StringWriter s = new StringWriter();
        PrintWriter  p = new PrintWriter( s );
        p.println( "thread=" + threadType + ",memType=" + memoryType + ",priority=" + priority + ",idDelay=" + iDelay + ",period=" + period + ",");
        return s.toString();
    }
}