/*
 * Created on Jun 7, 2004 
 * 
 *  Copyright 2004, by the California Institute of Technology. ALL RIGHTS RESERVED. 
 *  United States Government Sponsorship acknowledged. Any commercial use must be 
 *  negotiated with the Office of Technology Transfer at the California Institute 
 *  of Technology. 
 *
 *  This software may be subject to U.S. export control laws and regulations.  
 *  By accepting this document, the user agrees to comply with all applicable U.S. 
 *  export laws and regulations.  User has the responsibility to obtain export 
 *  licenses, or other export authority as may be required before exporting such 
 *  information to foreign countries or providing access to foreign persons. 
 */
package suramadu.util.exec;

import java.io.PrintWriter;
import java.io.StringWriter;

/**
 * @author dan To change the template for this generated type comment go to Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code
 *         and Comments
 */
public class ThreadLauncherData {
    public final static int        NOT_SET              = -2;

    private long                   iDelay               = NOT_SET;

    /**
     * 
     * @uml.property name="period"
     */
    private long period = NOT_SET;


    /**
     * 
     * @uml.property name="threadType"
     * @uml.associationEnd 
     * @uml.property name="threadType" multiplicity="(0 1)"
     */
    private ThreadType threadType = null;

    /**
     * 
     * @uml.property name="thread"
     */
    private Thread thread = null;


    /**
     * 
     * @uml.property name="memoryType"
     * @uml.associationEnd 
     * @uml.property name="memoryType" multiplicity="(0 1)"
     */
    private MemoryType memoryType = null;

    /**
     * 
     * @uml.property name="iterations"
     */
    private int iterations = NOT_SET;

    /**
     * 
     * @uml.property name="runnable"
     */
    private Runnable runnable;

    /**
     * 
     * @uml.property name="priority"
     */
    private int priority;


    /**
     * Creates a new instance of Launcher Data
     * 
     * @param runnable
     *            the object to run in the created thread. The deafult
     */
    public ThreadLauncherData( Runnable runnable ) {
        this.runnable = runnable;
    }

    /**
     * Creates a new instance of Launcher Data
     * 
     * @param runnable
     *            the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     */
    public ThreadLauncherData( Runnable runnable, int priority ) {
        this( runnable );
        this.priority = priority;
    }

    /**
     * Creates a new instance of Launcher Data
     * 
     * @param runnable
     *            the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     */
    public ThreadLauncherData( Runnable runnable, int priority, long period ) {
        this( runnable, priority );
        setPeriod( period );
    }

    /**
     * Creates a new instance of Launcher Data
     * 
     * @param runnable
     *            the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     * @param initialDelay
     *            the initialDelay in milliseconds.
     */
    public ThreadLauncherData( Runnable runnable, int priority, long period, long initialDelay ) {
        this( runnable, priority, period );
        setInitialDelay( initialDelay );
    }

    /**
     * Creates a new instance of Launcher Data
     * 
     * @param tt
     *            the ThreadType of the thread to create.
     * @param runnable
     *            Runnable the object to run in the created thread. The deafult Launcher.ThreadType is REALTIMETHREAD the deafault
     *            MemoryArea is Heap and the deafult priority is PriorityScheduler.MAX_PRIORITY.
     */
    public ThreadLauncherData( ThreadType tt, Runnable runnable ) {
        this( runnable );
        setThreadType( tt );
    }

    /**
     * Creates a new instance of Launcher Data
     * 
     * @param tt
     *            the ThreadType of the thread to create.
     * @param runnable
     *            Runnable the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     */
    public ThreadLauncherData( ThreadType tt, Runnable runnable, int priority ) {
        this( runnable, priority );
        setThreadType( tt );
    }

    /**
     * Creates a new instance of Launcher Data
     * 
     * @param tt
     *            the ThreadType of the thread to create.
     * @param runnable
     *            the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     */
    public ThreadLauncherData( ThreadType tt, Runnable runnable, int priority, long period ) {
        this( runnable, priority, period );
        setThreadType( tt );
    }

    /**
     * Creates a new instance of Launcher Data
     * 
     * @param tt
     *            the ThreadType of the thread to create.
     * @param runnable
     *            the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     * @param initialDelay
     *            the initialDelay in milliseconds.
     */
    public ThreadLauncherData( ThreadType tt, Runnable runnable, int priority, long period, long initialDelay ) {
        this( runnable, priority, period, initialDelay );
        setThreadType( tt );
    }

    /**
     * Creates a new instance of Launcher Data
     * 
     * @param tt
     *            the ThreadType of the thread to create.
     * @param mt
     *            the MemoryType in which to execute the created thread.
     * @param runnable
     *            the object to run in the created thread. The deafult Launcher.ThreadType is REALTIMETHREAD the deafault MemoryArea
     *            is Heap and the deafult priority is PriorityScheduler.MAX_PRIORITY.
     */
    public ThreadLauncherData( ThreadType tt, MemoryType mt, Runnable runnable ) {
        this( tt, runnable );
        setMemoryType( mt );
    }

    /**
     * Creates a new instance of Launcher
     * 
     * @param tt
     *            the ThreadType of the thread to create.
     * @param mt
     *            the MemoryType in which to execute the created thread.
     * @param runnable
     *            the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     */
    public ThreadLauncherData( ThreadType tt, MemoryType mt, Runnable runnable, int priority ) {
        this( tt, runnable, priority );
        setMemoryType( mt );
    }

    /**
     * Creates a new instance of Launcher Data
     * 
     * @param tt
     *            the ThreadType of the thread to create.
     * @param mt
     *            the MemoryType in which to execute the created thread.
     * @param runnable
     *            the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     */
    public ThreadLauncherData( ThreadType tt, MemoryType mt, Runnable runnable, int priority, long period ) {
        this( tt, runnable, priority, period );
        setMemoryType( mt );
    }

    /**
     * Creates a new instance of Launcher Data
     * 
     * @param tt
     *            the ThreadType of the thread to create.
     * @param mt
     *            the MemoryType in which to execute the created thread.
     * @param runnable
     *            the object to run in the created thread.
     * @param priority
     *            the priority of the thread in question. the deafult priority is PriorityScheduler.MAX_PRIORITY.
     * @param period
     *            the period in milliseconds.
     * @param initialDelay
     *            the initialDelay in milliseconds.
     */
    public ThreadLauncherData( ThreadType tt, MemoryType mt, Runnable runnable, int priority, long period, long initialDelay ) {
        this( tt, runnable, priority, period, initialDelay );
        setMemoryType( mt );
    }

    /**
     * Setters and getters for member fields
     * 
     * @uml.property name="iterations"
     */
    public void setIterations(int iterations) {
        this.iterations = iterations;
    }

    /**
     * 
     * @uml.property name="iterations"
     */
    public int getIterations() {
        return iterations;
    }


    /**
     * 
     * @uml.property name="memoryType"
     */
    public void setMemoryType(MemoryType mt) {
        memoryType = mt;
    }

    /**
     * 
     * @uml.property name="memoryType"
     */
    public MemoryType getMemoryType() {
        return memoryType;
    }

    /**
     * 
     * @uml.property name="thread"
     */
    public Thread getThread() {
        return thread;
    }


    /**
     * 
     * @uml.property name="threadType"
     */
    public void setThreadType(ThreadType tt) {
        threadType = tt;
    }

    /**
     * 
     * @uml.property name="threadType"
     */
    public ThreadType getThreadType() {
        return threadType;
    }

    /**
     * 
     * @uml.property name="priority"
     */
    public void setPriority(int p) {
        priority = p;
    }

    /**
     * 
     * @uml.property name="priority"
     */
    public int getPriority() {
        return priority;
    }

    /**
     * 
     * @uml.property name="period"
     */
    public void setPeriod(long p) {
        period = p;
    }

    /**
     * 
     * @uml.property name="period"
     */
    public long getPeriod() {
        return period;
    }



    public void setInitialDelay( long d ) {
        iDelay = d;
    }

    public long getInitialDelay() {
        return iDelay;
    }

    /**
     * @return runnable
     * 
     * @uml.property name="runnable"
     */
    public Runnable getRunnable() {
        return runnable;
    }

    /**
     * @return a string representing the contents of this class.
     */
    public String toString() {
        StringWriter s = new StringWriter();
        PrintWriter  p = new PrintWriter( s );
        p.println( "thread=" + threadType + ",memType=" + memoryType + ",priority=" + priority + ",idDelay=" + iDelay + ",period=" + period + ",");
        return s.toString();
    }
}