/*
 * Created on Jun 7, 2004 
 * 
 *  Copyright 2004, by the California Institute of Technology. ALL RIGHTS RESERVED. 
 *  United States Government Sponsorship acknowledged. Any commercial use must be 
 *  negotiated with the Office of Technology Transfer at the California Institute 
 *  of Technology. 
 *
 *  This software may be subject to U.S. export control laws and regulations.  
 *  By accepting this document, the user agrees to comply with all applicable U.S. 
 *  export laws and regulations.  User has the responsibility to obtain export 
 *  licenses, or other export authority as may be required before exporting such 
 *  information to foreign countries or providing access to foreign persons. 
 */
package suramadu.util.exec;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.Iterator;

import javax.realtime.HeapMemory;
import javax.realtime.RealtimeThread;

/**
 * @author dan To change the template for this generated type comment go to Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code
 *         and Comments
 */
public class ThreadLauncherFactory {

    private final static String JAVA_THREADS_CLASS = "suramadu.util.exec.JavaThreadLauncher";
    private final static String RT_THREADS_CLASS   = "suramadu.util.exec.RTThreadLauncher";

    public static ThreadLauncher getLauncher( ThreadLauncherData data ) throws NullPointerException, ClassNotFoundException {
        Class launchClass = null;
        ThreadLauncher launcher = null;
        ThreadType type = data.getThreadType();

        if ( type.equals( ThreadType.JAVATHREAD ) || type.equals( ThreadType.PERIODICJAVATHREAD ) ) {
            launchClass = Class.forName( JAVA_THREADS_CLASS );
        }
        else if ( type.equals( ThreadType.REALTIMETHREAD ) || type.equals( ThreadType.NOHEAPREALTIMETHREAD ) ) {
            launchClass = Class.forName( RT_THREADS_CLASS );
        }

        try {
            launcher = (ThreadLauncher)launchClass.newInstance();
            launcher.setData( data );
        }
        catch ( Exception e ) {
            e.printStackTrace();
            throw new ClassNotFoundException( "Error accessing ThreadLauncher class: " );
        }
        return launcher;
    }
    
    public static void startTest( final Class testClass, final Class propsClass ) {
        try {
            System.out.println( "startTest: " + testClass + ", " + propsClass );
            Method method = propsClass.getMethod( "factory", new Class[] { Class.class } );
	        for ( final Iterator i = (Iterator)method.invoke( null, new Object[] { testClass } ); i.hasNext(); ) {
	            Thread t = new RealtimeThread() {
	                public void run() {
//	                    ImmortalMemory.instance().enter( new Runnable() {
	                    HeapMemory.instance().enter( new Runnable() {
	                        public void run() {
	                            try {
	                                Constructor c   = testClass.getConstructor( new Class[] { propsClass } );
	                                Object      o   = c.newInstance( new Object[] { i.next() } );
	                                Method      run = testClass.getMethod( "run", null );
	                                run.invoke( o, null );	                                
                                }
                                catch ( Exception e ) {
                                    e.printStackTrace();
                                }
	                        }
	                    } );
	                }
	            };
	            t.start();
	            t.join();
	        }
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }
}