/*
 * HistogramData.java
 *
 * Created on August 15, 2003, 6:54 AM
 * 
 *  Copyright 2004, by the California Institute of Technology. ALL RIGHTS RESERVED. 
 *  United States Government Sponsorship acknowledged. Any commercial use must be 
 *  negotiated with the Office of Technology Transfer at the California Institute 
 *  of Technology. 
 *
 *  This software may be subject to U.S. export control laws and regulations.  
 *  By accepting this document, the user agrees to comply with all applicable U.S. 
 *  export laws and regulations.  User has the responsibility to obtain export 
 *  licenses, or other export authority as may be required before exporting such 
 *  information to foreign countries or providing access to foreign persons. 
 */

package suramadu.util.stats;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * A class to facilitate the creation of a histogram data structure for examining the distribution
 * of various kinds of numeric data.
 * 
 * @author mbi
 */
public class Histogram {
    /**
     * Test method for unit testing. Should be commented out for shipping
     * 
     * @param args
     *            the usual main() args.
     */
    public static void main( String args[] ) {
        Collection c = new ArrayList();

        System.out.println();
        for ( int i = 0; i < args.length; i++ ) {
            c.add( new Integer( args[ i ] ) );
        }
        System.out.println();

        Histogram h = new Histogram( 0, 10, 1, c );

        System.out.println( h );
        System.out.println( "MODE = " + h.getMode() );
    }

    /**
     * A holder for this Histogram's specification
     * 
     * @see suramadu.util.stats.HistogramBinSpec
     * 
     * @uml.property name="binSpec"
     * @uml.associationEnd 
     * @uml.property name="binSpec" multiplicity="(0 1)"
     */
    private HistogramBinSpec binSpec;

    /**
     * Creates a new instance of Histogram with the specified low bound, high bound, and numBins,
     * with the specified Collection. Units are arbitrary.
     * 
     * @param loBound
     *            the lowest bound for this Histogram
     * @param hiBound
     *            the highest bound for this Histogram
     * @param numBins
     *            the number of HistogramBins to create in this Histogram
     * @param c
     *            a Collection of like Objects that implement the java.lang.Comparable interface to
     *            be collated into this Histogram.
     * @see java.lang.Comparable
     * @see java.util.Collection
     * @see suramadu.util.stats.HistogramBinSpec
     */
    public Histogram( int loBound, int hiBound, int numBins, Collection c ) {
        this( new HistogramBinSpec( loBound, hiBound, numBins ), c );
    }

    /**
     * Creates a new instance of Histogram with the specified low bound, high bound, and numBins,
     * with the specified Collection. Units are arbitrary.
     * 
     * @param loBound
     *            the lowest bound for this Histogram
     * @param hiBound
     *            the highest bound for this Histogram
     * @param numBins
     *            the number of HistogramBins to create in this Histogram
     * @param c
     *            a Collection of like Objects that implement the java.lang.Comparable interface to
     *            be collated into this Histogram.
     * @see java.lang.Comparable
     * @see java.util.Collection
     * @see suramadu.util.stats.HistogramBinSpec
     */
    public Histogram( short loBound, short hiBound, short numBins, Collection c ) {
        this( new HistogramBinSpec( loBound, hiBound, numBins ), c );
    }

    /**
     * Creates a new instance of Histogram with the specified low bound, high bound, and numBins,
     * with the specified Collection. Units are arbitrary.
     * 
     * @param loBound
     *            the lowest bound for this Histogram
     * @param hiBound
     *            the highest bound for this Histogram
     * @param numBins
     *            the number of HistogramBins to create in this Histogram
     * @param c
     *            a Collection of like Objects that implement the java.lang.Comparable interface to
     *            be collated into this Histogram.
     * @see java.lang.Comparable
     * @see java.util.Collection
     * @see suramadu.util.stats.HistogramBinSpec
     */
    public Histogram( byte loBound, byte hiBound, byte numBins, Collection c ) {
        this( new HistogramBinSpec( loBound, hiBound, numBins ), c );
    }

    /**
     * Creates a new instance of Histogram with the specified low bound, high bound, and numBins,
     * with the specified Collection. Units are arbitrary.
     * 
     * @param loBound
     *            the lowest bound for this Histogram
     * @param hiBound
     *            the highest bound for this Histogram
     * @param numBins
     *            the number of HistogramBins to create in this Histogram
     * @param c
     *            a Collection of like Objects that implement the java.lang.Comparable interface to
     *            be collated into this Histogram.
     * @see java.lang.Comparable
     * @see java.util.Collection
     * @see suramadu.util.stats.HistogramBinSpec
     */
    public Histogram( long loBound, long hiBound, long numBins, Collection c ) {
        this( new HistogramBinSpec( loBound, hiBound, numBins ), c );
    }

    /**
     * Creates a new instance of Histogram with the specified low bound, high bound, and numBins,
     * with the specified Collection. Units are arbitrary.
     * 
     * @param loBound
     *            the lowest bound for this Histogram
     * @param hiBound
     *            the highest bound for this Histogram
     * @param numBins
     *            the number of HistogramBins to create in this Histogram
     * @param c
     *            a Collection of like Objects that implement the java.lang.Comparable interface to
     *            be collated into this Histogram.
     * @see java.lang.Comparable
     * @see java.util.Collection
     * @see suramadu.util.stats.HistogramBinSpec
     */
    public Histogram( float loBound, float hiBound, float numBins, Collection c ) {
        this( new HistogramBinSpec( loBound, hiBound, numBins ), c );
    }

    /**
     * Creates a new instance of Histogram with the specified low bound, high bound, and numBins,
     * with the specified Collection. Units are arbitrary.
     * 
     * @param loBound
     *            the lowest bound for this Histogram
     * @param hiBound
     *            the highest bound for this Histogram
     * @param numBins
     *            the number of HistogramBins to create in this Histogram
     * @param c
     *            a Collection of like Objects that implement the java.lang.Comparable interface to
     *            be collated into this Histogram.
     * @see java.lang.Comparable
     * @see java.util.Collection
     * @see suramadu.util.stats.HistogramBinSpec
     */
    public Histogram( double loBound, double hiBound, double numBins, Collection c ) {
        this( new HistogramBinSpec( loBound, hiBound, numBins ), c );
    }

    /**
     * Creates a new instance of Histogram using the specified HistogramBinSpec
     * 
     * @param binSpec
     *            an instance of a HistogramBinSpec for this Histogram
     * @see suramadu.util.stats.HistogramBinSpec
     */
    public Histogram( HistogramBinSpec binSpec ) {
        this.binSpec = binSpec;
    }

    /**
     * Creates a new instance of Histogram using the specified HistogramBinSpec with the specified
     * Collection. Units are arbitrary.
     * 
     * @param binSpec
     *            an instance of a HistogramBinSpec for this Histogram
     * @param c
     *            a Collection of like Objects that implement the java.lang.Comparable interface
     * @see java.lang.Comparable
     * @see java.util.Collection
     * @see suramadu.util.stats.HistogramBinSpec
     */
    public Histogram( HistogramBinSpec binSpec, Collection c ) {
        this( binSpec );
        for ( Iterator i = c.iterator(); i.hasNext(); ) {
            add( (Number)i.next() );
        }
    }

    /**
     * Creates a new instance of Histogram using an array of values organized as pairs of bounds,
     * the even numbered elements being low bounds, and the odd numbered elements being high bounds
     * (zero based). Bin consistency is checked at construction time.
     * 
     * @param bounds
     *            an array of bounds, alternating between low bound and high bound for consecutive
     *            and adjascent HistogramBins.
     */
    public Histogram( Comparable[] bounds ) {
        this( new HistogramBinSpec( bounds ) );
    }

    /**
     * Creates a new instance of Histogram with the specified Collection. The bounds are specified
     * using an array of values organized as pairs of bounds, the even numbered elements being low
     * bounds, and the odd numbered elements being high bounds(zero based). Bin consistency is
     * checked at construction time.
     * 
     * @param bounds
     *            an array of bounds, alternating between low bound and high bound for consecutive
     *            and adjascent HistogramBins.
     * @param c
     *            a Collection of like Objects that implement the java.lang.Comparable interface
     */
    public Histogram( Comparable[] bounds, Collection c ) {
        this( new HistogramBinSpec( bounds ), c );
    }

    /**
     * Creates a new instance of Histogram with the specified array of Number Objects. The bounds
     * are specified using an array of values organized as pairs of bounds, the even numbered
     * elements being low bounds, and the odd numbered elements being high bounds(zero based). Bin
     * consistency is checked at construction time.
     * 
     * @param bounds
     *            an array of bounds, alternating between low bound and high bound for consecutive
     *            and adjascent HistogramBins.
     * @param nArray
     *            an array of like Number Objects.
     */
    public Histogram( Comparable bounds[], Number[] nArray ) {
        this( bounds );
        for ( int i = 0; i < nArray.length; i++ ) {
            add( nArray[ i ] );
        }
    }

    /**
     * Adds a Number Object to this Histogram
     * 
     * @param n
     *            the Number object to add.
     */
    public synchronized void add( Number n ) {
        binSpec.getBin( n ).add( n );
    }

    /**
     * Gets the HistogramBin from this Histogram that contains the most values.
     * 
     * @return the HistogramBin from this Histogram that contains the most values.
     * @see suramadu.util.stats.HistogramBin
     */
    public HistogramBin getMode() {
        return binSpec.getModeBin();
    }

    /**
     * Creates a String representation for this Histogram.
     * 
     * @return a String representation for this Histogram.
     */
    public String toString() {
        StringWriter s = new StringWriter();
        PrintWriter p = new PrintWriter( s );

        for ( Iterator i = binSpec.iterator(); i.hasNext(); ) {
            p.println( i.next() );
        }
        return s.toString();
    }
}