/*
 * Range.java
 *
 * Created on August 19, 2003, 5:04 PM
 * 
 *  Copyright 2004, by the California Institute of Technology. ALL RIGHTS RESERVED. 
 *  United States Government Sponsorship acknowledged. Any commercial use must be 
 *  negotiated with the Office of Technology Transfer at the California Institute 
 *  of Technology. 
 *
 *  This software may be subject to U.S. export control laws and regulations.  
 *  By accepting this document, the user agrees to comply with all applicable U.S. 
 *  export laws and regulations.  User has the responsibility to obtain export 
 *  licenses, or other export authority as may be required before exporting such 
 *  information to foreign countries or providing access to foreign persons. 
 */

package suramadu.util.stats;

/**
 * An interface representing a range of values, and a counter to tally the number of values that
 * fall within this range. It is used to abstract a bin in a histogram.
 * 
 * @author mbi
 */

public interface Range {

    /**
     * Get the high bound of the range. This is the value that is <b>greater </b> than all values in
     * the range.
     * 
     * @return an object that implements the Comparable interface. This is usually a subclass of the
     *         Number object, such as Integer, Float, Double, etc.
     * 
     * @uml.property name="hiBound"
     */
    public Comparable getHiBound();

    /**
     * Get the lower bound of the range. This is the value that is <b>less than or equal to </b> all
     * values in the range.
     * 
     * @return an object that implements the Comparable interface. This is usually a subclass of the
     *         Number object, such as Integer, Float, Double, etc.
     * 
     * @uml.property name="loBound"
     */
    public Comparable getLoBound();

    /**
     * Get the count of the number of values that fall within this range.
     * 
     * @return an object that implements the Comparable interface. This is usually a subclass of the
     *         Number object, such as Integer, Float, Double, etc.
     * @see #getHiBound()
     * @see #getLoBound()
     */
    public long getCount();

    /**
     * A helper method that will merge this range with another range iff the ranges are adjescent.
     * 
     * @param range
     *            the Range object to merge with this range.
     * @return a new Object that implements the Range interface which encompasses both the current
     *         Range, and the specified Range. If the two ranges are not adjascent, then
     *         &lt;null&gt; is returned.
     */
    public Range merge( Range range );
}