/*
 *  Copyright 2004, by the California Institute of Technology. ALL RIGHTS RESERVED. 
 *  United States Government Sponsorship acknowledged. Any commercial use must be 
 *  negotiated with the Office of Technology Transfer at the California Institute 
 *  of Technology. 
 *
 *  This software may be subject to U.S. export control laws and regulations.  
 *  By accepting this document, the user agrees to comply with all applicable U.S. 
 *  export laws and regulations.  User has the responsibility to obtain export 
 *  licenses, or other export authority as may be required before exporting such 
 *  information to foreign countries or providing access to foreign persons. 
 */

#ifndef suramadu_util_exec_getTimeRegister_h
#define suramadu_util_exec_getTimeRegister_h

#ifdef __cplusplus
#include <iostream>
#else // __cplusplus
#include <stdio.h>
#endif // __cplusplus

#ifdef DARWIN
#include "/usr/include/gcc/darwin/default/ppc_intrinsics.h"
#endif

#if defined(ppc) || defined(PPC) || defined(powerpc) ||defined(_ARCH_PPC) ||defined(__PPC__) || defined(__ppc__) || defined(__PowerPC__) || defined(__powerpc__)
#ifndef GG_POWER_PC
#define GG_POWER_PC=ppc
#endif
#elif defined(i386) || defined(__i386) || defined(__i386__)
#ifndef GG_X86
#define GG_X86 x86
#endif
#elif defined(i486) || defined(__i486) || defined(__i486__)
#ifndef GG_X86
#define GG_X86 x86
#endif
#elif defined(i586) || defined(__i586) || defined(__i586__)
#ifndef GG_X86
#define GG_X86 x86
#endif
#elif defined(i686) || defined(__i686) || defined(__i686__)
#ifndef GG_X86
#define GG_X86 x86
#endif
#elif defined(i786) || defined(__i786) || defined(__i786__)
#ifndef GG_X86
#define GG_X86 x86
#endif
#elif defined(ix86) || defined(_ix86_) || defined(__ix86__)
#ifndef GG_X86
#define GG_X86 x86
#endif
#elif defined(x86) || defined(_x86_) || defined(__x86__)
#ifndef GG_X86
#define GG_X86 x86
#endif
#endif

#if 0
#ifdef __cplusplus
#ifdef ghs
	    // Greenhills can't print long longs!  DUH!
	    std::ostream& operator<<(std::ostream& os, unsigned long long ull) {
		char buffer[100];
		buffer[99] = 0;
		char* c = buffer+98;
		
		unsigned long long tmp;
		do {
		    tmp = ull / 10;
		    tmp *= 10;
		    tmp = ull - tmp;
		    *c-- = '0' + (char)tmp;
		    ull /= 10;
		}
		while (ull != 0);
		return (os << (c+1));
	    }
#endif // ghs
#endif // __cplusplus
#endif // 0

/**
*  These macros were lifted from the file /usr/include/asm/time.h
*/
#ifdef GG_POWER_PC

#ifdef ghs  // the green hills compiler

// for VxWorks (Greenhills)
unsigned long long int getCyclesInternal() {
    register unsigned long long cycles = 0;

#pragma asm
again:
        mftbu   0
        mftb    11
        mftbu   12
        cmpw    0,12
        bne     again
#pragma endasm
// Even a printf of cycles at this point causes this function to return 0!
	    return cycles;
}

// Greenhills and PPC => for VxWorks
inline unsigned long long int getCycles() {
    unsigned long long int tmp    = getCyclesInternal();
    unsigned long long int cycles = tmp >> 32;
    cycles |= (tmp << 32);

#if __DEBUGGING_STUFF
    unsigned long *cyclesL = (unsigned long*)&tmp;
    unsigned long *cyclesH = cyclesL;
    ++cyclesH;
    unsigned long long int cycles2 = (unsigned long long)(*cyclesH);
    cycles2 *= 0x100000000ULL;
    cycles2 += (unsigned long long)*cyclesL;
#ifdef __cplusplus
    if (cycles != cycles2) { std::cerr <<"NOT EQUAL:" << cycles << ", " << cycles2 << std::endl; }

    std::cout << std::hex << "raw: " << tmp << ", cyclesH: " << *cyclesH << ", cyclesL: " << *cyclesL;
    std::cout << ", cycles: " << cycles << std::dec << std::endl;
#else // __cplusplus
    printf("raw: %xll, cyclesH: %Xll, cyclesL: %Xll, cycles: %Xll\n", tmp, *cyclesH, *cyclesL, cycles);
#endif // __cplusplus
#endif // __DEBUGGING_STUFF

    return cycles;
}
#else // ghs (Greenhills)

#ifdef DARWIN

// for Macintosh OS X
inline unsigned long long int getCycles() {
    unsigned long int lo, hi, check;

    do {
        hi    = __mftbu();
        lo    = __mftb();
        check = __mftbu();
    } while ( hi != check );
    return( (unsigned long long int)hi << 32) | lo;
}

#else   // DARWIN

// for Linux
inline unsigned long long int getCycles() {
    unsigned long long int cycles = 0;

    asm volatile( "\
1:\n\
        mftbu   %[upper]\n\
        mftb    %[lower]\n\
        mftbu   %0\n\
        cmpw    %0, %[upper]\n\
        bne     1\n"
            : [upper] "=r" (*(((long *)&cycles)+0)),
              [lower] "=r" (*(((long *)&cycles)+1))
            :
            : "cr0" ); 
    return cycles;
}

#endif // DARWIN

#endif // ghs (Greenhills)

#else  // GG_POWER_PC

#ifdef GG_X86

inline unsigned long long int getCycles() {
    unsigned long long int cycles = 0;
    asm volatile ( "rdtsc" : "=A" (cycles) );
    return cycles;
}

#else // GG_X86
#include <sys/time.h>

// Sun - get cycles not really available, so use sparc high res timer:
inline unsigned long long int getCycles() {
        hrtime_t now = gethrtime();     // Units are nanoseconds.
    return (unsigned long long)now;
}
#endif // GG_X86

#endif // GG_POWER_PC

#endif // file tag
