/*
 *  Copyright 2004, by the California Institute of Technology. ALL RIGHTS RESERVED.
 *  United States Government Sponsorship acknowledged. Any commercial use must be
 *  negotiated with the Office of Technology Transfer at the California Institute
 *  of Technology.
 *
 *  This software may be subject to U.S. export control laws and regulations.
 *  By accepting this document, the user agrees to comply with all applicable U.S.
 *  export laws and regulations.  User has the responsibility to obtain export
 *  licenses, or other export authority as may be required before exporting such
 *  information to foreign countries or providing access to foreign persons.
 */
                                                                                                        
#include <jni.h>
#include "getTimeRegister.h"
#include "suramadu_util_time_Time.h"
#include "estimate_GHz.h"
#include <stdio.h>
#include <time.h>

#ifndef DARWIN
#include <sys/resource.h>
#endif // DARWIN

#include <sys/types.h>

// returns the current cycle count
/*
 * Class:     suramadu_util_Time
 * Method:    getCycles
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_suramadu_util_time_Time_getCycles( JNIEnv *env, jclass obj ) {
    return getCycles();
}

/*
 * Class:     suramadu_util_time_Time
 * Method:    getCPS
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_suramadu_util_time_Time_getCPS( JNIEnv *env, jclass obj, jint bounds, jint loops, jint tolerance, jint tolerance_retries, jstring output ) {
    jlong retVal = 0;
    const char *filename = (*env)->GetStringUTFChars( env, output, NULL );
    if ( NULL == filename ) return (jlong)NULL;
    retVal = estimate_GHz( bounds, loops, tolerance, tolerance_retries, filename );
    (*env)->ReleaseStringUTFChars( env, output, filename );
    return retVal;
}

// returns the current high resolution time
/*
 * Class:     suramadu_util_Time
 * Method:    getTime
 * Signature: ([J)V
 */
JNIEXPORT void JNICALL Java_suramadu_util_time_Time_getTime( JNIEnv *env, jclass obj, jlongArray timeArray ) {
    jlong* temp = (*env)->GetLongArrayElements( env, timeArray, 0 );

#ifdef DARWIN
    struct timeval current;
    gettimeofday( &current, NULL );
    temp[ 1 ] = current.tv_usec;
#else   // !DARWIN
    struct timespec current;
    clock_gettime( CLOCK_REALTIME, &current );
    temp[ 1 ] = current.tv_nsec;
#endif  // DARWIN

    temp[ 0 ] = current.tv_sec;
    (*env)->ReleaseLongArrayElements( env, timeArray, temp, 0 );
}

