/*
 * SuramaduTestStub
 *
 *  Copyright 2004, by the California Institute of Technology. ALL RIGHTS RESERVED.
 *  United States Government Sponsorship acknowledged. Any commercial use must be
 *  negotiated with the Office of Technology Transfer at the California Institute
 *  of Technology.
 *
 *  This software may be subject to U.S. export control laws and regulations.
 *  By accepting this document, the user agrees to comply with all applicable U.S.
 *  export laws and regulations.  User has the responsibility to obtain export
 *  licenses, or other export authority as may be required before exporting such
 *  information to foreign countries or providing access to foreign persons.
 */

package suramadu.tests;

import java.util.Iterator;

import suramadu.util.exec.MemoryType;
import suramadu.util.exec.PropertyValidationException;
import suramadu.util.exec.TestInitializer;
import suramadu.util.exec.TestProperties;
import suramadu.util.exec.ThreadLauncher;
import suramadu.util.exec.ThreadLauncherData;
import suramadu.util.exec.ThreadLauncherFactory;
import suramadu.util.exec.ThreadType;
import suramadu.util.stats.Range;
import suramadu.util.stats.StatPack;
import suramadu.util.time.Time;

/**
 * This is a stubbed that demonstrates how to build a Suramadu benchmark.
 * <p>
 * <B><I>PURPOSE: </I> </B> 
 * <p>
 * <B><I>DESIGN: </I> </B> 
 * <p>
 * <B><I>ASSUMPTIONS: </I> </B> 
 * <p>
 * <B><I>INPUTS: </I> </B> <br>
 * <table>
 * <tr>
 * <th align="left">Property</th>
 * <th align="right">Default Value</th>
 * <th align="left">Description</th>
 * </tr>
 * <tr>
 * <td>"iterations"</td>
 * <td>"1000"</td>
 * <td></td>
 * </tr>
 * <tr>
 * <td>"priority"</td>
 * <td>"250"</td>
 * <td></td>
 * </tr>
 * <tr>
 * <td>"period"</td>
 * <td>"1000"</td>
 * <td></td>
 * </tr>
 * <tr>
 * <td>"idelay"</td>
 * <td>"750"</td>
 * <td></td>
 * </tr>
 * <tr>
 * <td>"bgpriority"</td>
 * <td>"250"</td>
 * <td></td>
 * </tr>
 * <tr>
 * <td>"bgperiod"</td>
 * <td>"100"</td>
 * <td></td>
 * </tr>
 * <tr>
 * <td>"bgidelay"</td>
 * <td>"400"</td>
 * <td></td>
 * </tr>
 * <tr>
 * <td>"backgroundthreads"</td>
 * <td>"0"</td>
 * <td></td>
 * </tr>
 * </table>
 * <p>
 * <B><I>OUTPUTS: </I> </B>
 * <p>
 * <B><I>EXAMPLES: </I> </B>
 * <p>
 * <B><I>ORIGINS: </I> </B>
 * <p>
 * 
 * @author Mark Indictor
 */
public class SuramaduTestStub implements Runnable {
	
	public static class TestProps extends TestProperties {
	    public static final String[] DEFAULT_PROPERTIES = {
	    		"memtype",    "HEAP", 
				"threadtype", "RT", 
				"iterations", "10", 
				"priority",   "250", 
				"period",     "250", 
				"idelay",     "750", 
				"bgthreads",  "0", 
				"bgpriority", "250", 
				"bgperiod",   "100", 
				"bgidelay",   "400", 
				"histbins",   "50",
	    };
	    
	    public static Iterator factory( Class aClass ) throws PropertyValidationException, InstantiationException, IllegalAccessException {
	        return factory( TestProps.class, aClass, DEFAULT_PROPERTIES );
	    }

		/**
		 * @throws PropertyValidationException
		 */
		public TestProps() throws PropertyValidationException {
			super();
		}

		/**
		 * @param aClass
		 * @throws PropertyValidationException
		 */
		public TestProps(Class aClass) throws PropertyValidationException {
			super( aClass, DEFAULT_PROPERTIES );
		}

	    /**
	     * Saves argument values in static class members.
	     * 
	     * @throws NumberFormatException
	     *             If a property is misformed.
	     */
	    protected void loadArguments() throws PropertyValidationException {
	        threadType = getInputProperty( "threadtype" );
	        memType = getInputProperty( "memtype" );
	        iterations = getIntInputProperty( "iterations" );
	        priority = getIntInputProperty( "priority" );
	        period = getIntInputProperty( "period" );
	        idelay = getIntInputProperty( "idelay" );
	        bgThreads = getIntInputProperty( "bgthreads" );
	        bgPriority = getIntInputProperty( "bgpriority" );
	        bgPeriod = getIntInputProperty( "bgperiod" );
	        bgIdelay = getIntInputProperty( "bgidelay" );
	        histBins = getIntInputProperty( "histbins" );
	    }

	    /**
	     * Saves argument values in static class members.
	     * 
	     * @throws NumberFormatException
	     *             If a property is misformed.
	     */
	    protected void validateArguments() throws PropertyValidationException {
	        boolean ok = true;

	        if ( 0 > iterations ) {
	            ok = false;
	            System.out.println( "iterations property must be >= 0. Using default: " + iterations );
	        }
	        if ( 0 > priority ) {
	            ok = false;
	            System.out.println( "priority property must be >= 0. Using default: " + priority );
	        }
	        if ( 0 > iterations ) {
	            ok = false;
	            System.out.println( "period property must be >= 0. Using default: " + period );
	        }
	        if ( 0 > idelay ) {
	            ok = false;
	            System.out.println( "idelay property must be >= 0. Using default: " + idelay );
	        }
	        if ( 0 > bgThreads ) {
	            ok = false;
	            System.out.println( "bgthreads property must be >= 0. Using default: " + bgThreads );
	        }
	        if ( 0 > bgPriority ) {
	            ok = false;
	            System.out.println( "bgpriority property must be >= 0. Using default: " + bgPriority );
	        }
	        if ( 0 > bgPeriod ) {
	            ok = false;
	            System.out.println( "bgperiod property must be >= 0. Using default: " + bgPeriod );
	        }
	        if ( 0 > bgIdelay ) {
	            ok = false;
	            System.out.println( "bgidelay property must be >= 0. Using default: " + bgIdelay );
	        }
	        if ( !ok ) throw new PropertyValidationException( "One or more input properties are invalid." );
	    }

	    /**
	     * actual properties used by this test run. Filled in from default and specified properties at class instantiation.
	     */
	    /** retrieved from default or specified properties */
	    public static String         threadType;
	
	    /** retrieved from default or specified properties */
	    public static String         memType;
	
	    /** retrieved from default or specified properties. */
	    public static int            iterations;
	
	    /** retrieved from default or specified properties. */
	    public static int            priority;
	
	    /** retrieved from default or specified properties. */
	    public static long           period;
	
	    /** retrieved from default or specified properties. */
	    public static long           idelay;
	
	    /** retrieved from default or specified properties. */
	    public static int            bgPriority;
	
	    /** retrieved from default or specified properties. */
	    public static long           bgPeriod;
	
	    /** retrieved from default or specified properties. */
	    public static long           bgIdelay;
	
	    /** retrieved from default or specified properties. */
	    public static int            bgThreads;
	
	    /** the number of histogram bins to use for mode calculation (evenly divided between min and max) */
	    public static int            histBins;
	}

    /**
     * Local data for test: generated or otherwise calculated. Not from test properties.
     * 
     * @uml.property name="testProps"
     * @uml.associationEnd 
     * @uml.property name="testProps" multiplicity="(1 1)"
     */
    public TestProps testProps;

    /**
     * Thread type to use.
     * 
     * @uml.property name="tt"
     * @uml.associationEnd 
     * @uml.property name="tt" multiplicity="(0 1)"
     */
    public ThreadType tt;

    /**
     * Memory type to use
     * 
     * @uml.property name="mt"
     * @uml.associationEnd 
     * @uml.property name="mt" multiplicity="(0 1)"
     */
    public MemoryType mt;

    /** array for collecting timestamps during test */
    public long           timeArray[];

    /** iteration counter */
    public  int           counter;

	/**
     * constructor for test class Instantiates testProperties.
	 * @throws Throwable
	 * @throws IllegalAccessException
	 * @throws InstantiationException
     */
    public SuramaduTestStub() throws Exception {
        this( new TestProps( SuramaduTestStub.class ) );
    }

    public SuramaduTestStub( TestProps props ) throws Exception {
        System.out.println( props.getProperty( "propertiesfile" ) );
        testProps = props;
        counter   = 0;
    }
    
    /**
     * @param args
     *            the command line arguments
     */
    public static void main( String args[] ) {
        try {
	        for ( Iterator i = TestProps.factory( SuramaduTestStub.class ); i.hasNext(); ) {
	            Thread t = new Thread( new SuramaduTestStub( (TestProps)i.next() ) );
	            t.start();
	            t.join();
	        }
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    public void run() {
        try {
            /**
             * Since we need iterations + 1 number of measurements to get iterations number of timings, increment iterations before
             * using it.
             */
        	TestProps.iterations++;

            ThreadLauncher[] bgThreadArray = null;
            if ( 0 < TestProps.bgThreads ) {
                bgThreadArray = new ThreadLauncher[ TestProps.bgThreads ];
                for ( int i = 0; i < TestProps.bgThreads; i++ ) {
                    double randomDelay = Math.random() * TestProps.bgIdelay;
                    bgThreadArray[ i ] = ThreadLauncherFactory.getLauncher( new ThreadLauncherData( ThreadType.PERIODICJAVATHREAD, MemoryType.HEAPMEMORY, new BGThread(), TestProps.bgPriority, TestProps.bgPeriod, TestProps.bgIdelay ) );
                    bgThreadArray[ i ].launch();
                }
            }

            /** Can vary thread types used */
            tt = ThreadType.getThreadType( TestProps.threadType );

            /** Can vary memory types used */
            mt = MemoryType.getMemoryType( TestProps.memType );

            ThreadLauncher launcher = ThreadLauncherFactory.getLauncher( new ThreadLauncherData( tt, mt, new TestInit(), TestProps.priority, TestProps.period, TestProps.idelay ) );
            launcher.setIterations( TestProps.iterations );
            Thread t = launcher.launch();

            // wait for the test to complete (based on a calculation of how long it will take)
            t.join();

            // Kill off background threads.
            if ( null != bgThreadArray ) {
                for ( int i = 0; i < TestProps.bgThreads; i++ ) {
                    bgThreadArray[ i ].kill();
                }
            }

            // log results
            long[] diff = StatPack.diff( timeArray );
            long rtPeriod = Time.cyclesFromMs( TestProps.period );

            // convert differences into real-time jitter values.
            for ( int i = 0; i < diff.length; i++ ) {
                diff[ i ] = (long)Time.nsFromCycles( diff[ i ] - rtPeriod );
            }
            testProps.setOutputProperty( "rawdata", diff );
            testProps.setOutputProperty( "rawdatamax", StatPack.max( diff ) );
            testProps.setOutputProperty( "rawdatamin", StatPack.min( diff ) );
            testProps.setOutputProperty( "rawdatamean", StatPack.mean( diff ) );
            testProps.setOutputProperty( "rawdatastddev", StatPack.stddev( diff ) );
            testProps.setOutputProperty( "rawdatavariance", StatPack.variance( diff ) );

            Range mode = StatPack.mode( diff, TestProps.histBins );
            testProps.setOutputProperty( "rawdatamode.lobound", (Number)mode.getLoBound() );
            testProps.setOutputProperty( "rawdatamode.hibound", (Number)mode.getHiBound() );
            testProps.setOutputProperty( "rawdatamode.count",   mode.getCount() );
            testProps.setOutputProperty( "rawdatamode.percent", (double)mode.getCount() * 100.0 / ((double)TestProps.iterations - 1) );

            // report on what we're going to use.
            testProps.close();            ;
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    class TestInit extends TestInitializer {
        public void run() {
        }
    }

    class Test implements Runnable {
        public void run() {
        }
    }

    class BGThread implements Runnable {
        public void run() {
        }
    }
}