/*
 * ScopeMemoryTest.java Created on August 5, 2003, 11:40 AM
 * 
 *  Copyright 2004, by the California Institute of Technology. ALL RIGHTS RESERVED.
 *  United States Government Sponsorship acknowledged. Any commercial use must be
 *  negotiated with the Office of Technology Transfer at the California Institute
 *  of Technology.
 *
 *  This software may be subject to U.S. export control laws and regulations.
 *  By accepting this document, the user agrees to comply with all applicable U.S.
 *  export laws and regulations.  User has the responsibility to obtain export
 *  licenses, or other export authority as may be required before exporting such
 *  information to foreign countries or providing access to foreign persons.
 */

package suramadu.tests.memory;

import java.util.Iterator;
import javax.realtime.LTMemory;
import javax.realtime.PriorityScheduler;
import javax.realtime.ScopedMemory;
import suramadu.util.exec.MemoryType;
import suramadu.util.exec.PropertyValidationException;
import suramadu.util.exec.RTThreadLauncher;
import suramadu.util.exec.TestInitializer;
import suramadu.util.exec.TestProperties;
import suramadu.util.exec.ThreadLauncherFactory;
import suramadu.util.exec.ThreadType;
import suramadu.util.stats.StatPack;
import suramadu.util.time.Time;

/**
 * Tests the cost of entering and exiting ScopeMemory.
 * <p>
 * <B><I>PURPOSE: </I> </B> Tests the cost of entering and exiting ScopeMemory.
 * <p>
 * <B><I>DESIGN: </I> </B>
 * <p>
 * <B><I>ASSUMPTIONS: </I> </B>
 * <p>
 * <B><I>INPUTS: </I> </B> <table>
 * <tr>
 * <th align="left">Property</th>
 * <th align="right">Default Value</th>
 * <th align="left">Description</th>
 * </tr>
 * <tr>
 * <td>"iterations"</td>
 * <td>"1000000"</td>
 * <td>The number of test iterations to perform.</td>
 * </tr>
 * <tr>
 * <td>"priority"</td>
 * <td>"???"</td>
 * <td>???</td>
 * </tr>
 * <tr>
 * <td>"threadType"</td>
 * <td>"???"</td>
 * <td>???</td>
 * </tr>
 * </table>
 * <p>
 * <B><I>OUTPUTS: </I> </B>
 * <p>
 * <B><I>EXAMPLES: </I> </B>*
 * <p>
 * <B><I>ORIGINS: </I> </B>
 * <p>
 * 
 * @author briang
 * @version $Id: ScopeMemoryTest.java,v 1.1 2004/09/23 15:34:04 mbi Exp $
 */
public class ScopeMemoryTest implements Runnable {
	public static class TestProps extends TestProperties{
	    public static final String[] DEFAULT_PROPERTIES = {
				"iterations", "10", 
				"priority", "10", 
				"threadtype", "RT", 
				"memtype", "IMMORTAL",
	    };
	    
	    public static Iterator factory( Class aClass ) throws PropertyValidationException, InstantiationException, IllegalAccessException {
	        return factory( TestProps.class, aClass, DEFAULT_PROPERTIES );
	    }

		/**
		 * @throws PropertyValidationException
		 */
		public TestProps() throws PropertyValidationException {
			super();
		}

		/**
		 * @param aClass
		 * @throws PropertyValidationException
		 */
		public TestProps(Class aClass) throws PropertyValidationException {
			super( aClass, DEFAULT_PROPERTIES );
		}

	    /** Saves argument values in static class members.* */
	    protected void loadArguments() throws PropertyValidationException {
	        iterations = getIntInputProperty( "iterations" );
	        priority = getIntInputProperty( "priority" );
	        threadType = getInputProperty( "threadtype" );
	        memType = getInputProperty( "memtype" );
	    }

	    protected void validateArguments() throws PropertyValidationException {
	        boolean ok = true;

	        if ( !ok ) throw new PropertyValidationException( "One or more input properties are invalid." );
	    }

	    /**
	     * The number of itteration to perform
	     */
	    private static int           iterations        = 1000;

		/**
	     * The priority to run the test thread at.
	     */
	    private static int           priority          = PriorityScheduler.MAX_PRIORITY;

	    /**
	     * The type of thread to use 'RealtimeThread' or 'NoHeapRealtimeThread'
	     */
	    private static String        threadType;

	    /**
	     * The type of thread to use 'RealtimeThread' or 'NoHeapRealtimeThread'
	     */
	    private static String        memType;
	}
	
    /**
     * actual properties used by this test run. Filled in from default and specified properties at class instantiation.
     */
    public static TestProps 	 testProps;

    /**
     * The ScopeMemory to enter for testing.
     */
    private static ScopedMemory  scopedMemory1     = null;

    /**
     * The Inner ScopeMemory to enter for testing.
     */
    private static ScopedMemory  scopedMemory2     = null;

    /**
     * Stores [Iteration # Pre Scope Enter Time, Iteration # Post Scope Enter Time, .....]
     */
    private static long          enterArray[]      = null;

    /**
     * Stores [Iteration # Pre Scope Exit Time, Iteration # Post Scope Exit Time, .....]
     */
    private static long          exitArray[]       = null;

    /**
     * The current itteration
     */
    private static int           curIteration      = 0;

    /** Thread type to use. */
    public static ThreadType     tt;

    /** Memory type to use */
    public static MemoryType     mt;

	/**
     * constructor for test class Instantiates testProperties.
	 * @throws Throwable
	 * @throws IllegalAccessException
	 * @throws InstantiationException
     */
    public ScopeMemoryTest() throws Exception {
        this( new TestProps( ScopeMemoryTest.class ) );
    }

    public ScopeMemoryTest( TestProps props ) throws Exception {
        System.out.println( props.getProperty( "propertiesfile" ) );
        testProps = props;
    }
    
    /**
     * @param args
     *            the command line arguments
     */
    public static void main( String args[] ) {
        ThreadLauncherFactory.startTest( ScopeMemoryTest.class, TestProps.class );
    }

    public void run() {
        try {
            /** Can vary thread types used */
            tt = ThreadType.getThreadType( TestProps.threadType );

            /** Can vary memory types used */
            mt = MemoryType.getMemoryType( TestProps.memType );

            RTThreadLauncher immortalSetupLauncher = new RTThreadLauncher( ThreadType.REALTIMETHREAD, MemoryType.IMMORTALMEMORY, new ImmortalSetup(), TestProps.priority );
            immortalSetupLauncher.setIterations( 1 );
            Thread immortalSetupLauncherThread = immortalSetupLauncher.launch();

            // Wait for the thread to finish.
            try {
                immortalSetupLauncherThread.join();
            }
            catch ( InterruptedException ie ) {
                ie.printStackTrace();
            }

            // Display results
            final long enterDiffArray[] = new long[ TestProps.iterations ]; // Diff of Pre and Post Enter
            final long exitDiffArray[] = new long[ TestProps.iterations ]; // Diff of Pre and Post Exit

            // compute time differences
            for ( int i = 0; i < TestProps.iterations; i++ ) {
                enterDiffArray[ i ] = (long)Time.nsFromCycles( enterArray[ (i * 2) + 1 ] - enterArray[ i * 2 ] ); // * 2
                // compinsates
                // for
                // the
                // value
                // pairs
                exitDiffArray[ i ] = (long)Time.nsFromCycles( exitArray[ (i * 2) + 1 ] - exitArray[ i * 2 ] ); // * 2
                // compinsates
                // for
                // the
                // value
                // pairs
            }

            // output data.
            testProps.setOutputProperty( "scopeenter", enterDiffArray );
            testProps.setOutputProperty( "scopeexit", exitDiffArray );

            testProps.setOutputProperty( "enter.max", StatPack.max( enterDiffArray ) );
            testProps.setOutputProperty( "enter.min", StatPack.min( enterDiffArray ) );
            testProps.setOutputProperty( "enter.mean", StatPack.mean( enterDiffArray ) );
            testProps.setOutputProperty( "enter.stddev", StatPack.stddev( enterDiffArray ) );
            testProps.setOutputProperty( "enter.variance", StatPack.variance( enterDiffArray ) );

            testProps.setOutputProperty( "exit.max", StatPack.max( exitDiffArray ) );
            testProps.setOutputProperty( "exit.min", StatPack.min( exitDiffArray ) );
            testProps.setOutputProperty( "exit.mean", StatPack.mean( exitDiffArray ) );
            testProps.setOutputProperty( "exit.stddev", StatPack.stddev( enterDiffArray ) );
            testProps.setOutputProperty( "exit.variance", StatPack.variance( enterDiffArray ) );

            // report on what we're going to use.
            testProps.close();
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    /**
     * Used to configure and launch the test.
     */
    private static class Test implements Runnable {
        /**
         * Runs the rest.
         */
        public void run() {
            scopedMemory1.enter( new OuterScopeRunner() );
        }
    }

    /**
     * Used to configure and launch the test.
     */
    private static class OuterScopeRunner implements Runnable {
        /**
         * Runs the rest.
         */
        public void run() {
            InnerScopeRunner innerRunner = new InnerScopeRunner();
            for ( curIteration = 0; curIteration < TestProps.iterations; curIteration++ ) {
                enterArray[ curIteration * 2 ] = Time.getCycles(); // Pre Scope Enter
                scopedMemory2.enter( innerRunner );
                exitArray[ (curIteration * 2) + 1 ] = Time.getCycles(); // Post Scope Exit
            }
        }
    }

    /**
     * Used to configure and launch the test.
     */
    private static class InnerScopeRunner implements Runnable {
        /**
         * Runs the rest.
         */
        public void run() {
            long enterTime = Time.getCycles(); // end enter time
            enterArray[ (curIteration * 2) + 1 ] = enterTime; // Post Scope Enter
            //          System.out.println("End of Iteration # " + curIteration + " Step " +
            // (curIteration*2));
            exitArray[ (curIteration * 2) ] = Time.getCycles(); // Pre Scope Exit
        }
    }

    /**
     * Used to allocate Scope in immortal. This helps you avoid the newInstance(java.lang.reflect.Constructor c, java.lang.Object[]
     * args) method of MemoryArea. And you can allocate more than one object this way.
     */
    private static class ImmortalSetup extends TestInitializer {

        /**
         * Allocate Scope in immortal
         */
        public void run() {
            scopedMemory1 = new LTMemory( 1048576, 1048576 ); // 1M
            scopedMemory2 = new LTMemory( 1048576, 1048576 ); // 1M
            enterArray = new long[ TestProps.iterations * 2 ]; // * 2 since every iteration has an enter and
            // an exit
            exitArray = new long[ TestProps.iterations * 2 ]; // * 2 since every iteration has an enter and an
            // exit
            setTestRunner( new Test() );
        }
    }

}