/*
 * VMRuntimeSize.java Created on August 13, 2003, 12:57 PM
 *
 *  Copyright 2004, by the California Institute of Technology. ALL RIGHTS RESERVED.
 *  United States Government Sponsorship acknowledged. Any commercial use must be
 *  negotiated with the Office of Technology Transfer at the California Institute
 *  of Technology.
 *
 *  This software may be subject to U.S. export control laws and regulations.
 *  By accepting this document, the user agrees to comply with all applicable U.S.
 *  export laws and regulations.  User has the responsibility to obtain export
 *  licenses, or other export authority as may be required before exporting such
 *  information to foreign countries or providing access to foreign persons.
 */

package suramadu.tests.memory.footprint;

import java.util.Iterator;

import suramadu.util.exec.JavaThreadLauncher;
import suramadu.util.exec.PropertyValidationException;
import suramadu.util.exec.TestProperties;
import suramadu.util.exec.ThreadLauncherFactory;

/**
 * Measure the memory footprint of a Java application.
 * <p>
 * <B><I>PURPOSE: </I> </B>
 * <p>
 * <B><I>DESIGN: </I> </B> This object does not actually test the size of the VM. What it does instead is start ans stay active for
 * a configuaable amount of time so that an external shell script can measure memory usage.
 * <p>
 * <B><I>ASSUMPTIONS: </I> </B>
 * <p>
 * <B><I>INPUTS: </I> </B> <table>
 * <tr>
 * <th align="left">Property</th>
 * <th align="right">Default Value</th>
 * <th align="left">Description</th>
 * </tr>
 * <tr>
 * <td>"duration"</td>
 * <td>"3000"</td>
 * <td>The number of test iterations to perform.</td>
 * </tr>
 * </table>
 * <p>
 * <B><I>OUTPUTS: </I> </B>
 * <p>
 * <B><I>EXAMPLES: </I> </B>*
 * <p>
 * <B><I>ORIGINS: </I> </B>
 * <p>
 * 
 * @author briang
 */
public class VMRuntimeSize implements Runnable {
	public static class TestProps extends TestProperties{
	    public static final String[] DEFAULT_PROPERTIES = {
				"duration", "3000",
	    };
	    
	    public static Iterator factory( Class aClass ) throws PropertyValidationException, InstantiationException, IllegalAccessException {
	        return factory( TestProps.class, aClass, DEFAULT_PROPERTIES );
	    }

		/**
		 * @throws PropertyValidationException
		 */
		public TestProps() throws PropertyValidationException {
			super();
		}

		/**
		 * @param aClass
		 * @throws PropertyValidationException
		 */
		public TestProps(Class aClass) throws PropertyValidationException {
			super( aClass, DEFAULT_PROPERTIES );
		}

	    /**
	     * Saves argument values in static class members.
	     * 
	     * @throws NumberFormatException
	     *             If a property is misformed.
	     */
	    protected void loadArguments() throws PropertyValidationException {
            duration = getIntInputProperty( "duration" );
	    }

	    protected void validateArguments() throws PropertyValidationException {
	        boolean ok = true;

	        if ( 0 > duration ) {
	            ok = false;
	            System.out.println( "duration property must be >= 0. Using default: " + 3000 );
	        }
	        if ( !ok ) throw new PropertyValidationException( "One or more input properties are invalid." );
	    }

	    public static int   duration;
	}
	

    /**
     * Create local properties object based on properties file and default values
     */
    static TestProps testProps;


	/**
     * constructor for test class Instantiates testProperties.
	 * @throws Throwable
	 * @throws IllegalAccessException
	 * @throws InstantiationException
     */
    public VMRuntimeSize() throws Exception {
        this( new TestProps( VMRuntimeSize.class ) );
    }

    public VMRuntimeSize( TestProps props ) throws Exception {
        System.out.println( props.getProperty( "propertiesfile" ) );
        testProps = props;
    }
    
    /**
     * @param args
     *            the command line arguments
     */
    public static void main( String args[] ) {
        System.out.println( "JAVA APPLICATION START" );
        ThreadLauncherFactory.startTest( VMRuntimeSize.class, TestProps.class );
    }

    public void run() {
        try {
            JavaThreadLauncher launcher = new JavaThreadLauncher( new Test() );
            launcher.launch();
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    /**
     * The Runnable for this test.
     */
    private static class Test implements Runnable {
        /**
         * Just stays alive for duration
         */
        public void run() {
            Thread thread = Thread.currentThread();
            try {
                Thread.sleep( TestProps.duration );
            }
            catch ( InterruptedException ie ) {
                ie.printStackTrace();
            }
        }
    }
}